@@ -14,18 +14,22 @@ apply_patch_in_memory = _rust_module.py_apply_patch_in_memory
 parse_patch = _rust_module.py_parse_patch
 get_tool_instructions = _rust_module.py_get_tool_instructions
 get_api_instructions = _rust_module.py_get_api_instructions
+generate_patch = _rust_module.py_generate_patch
+generate_patch_from_files = _rust_module.py_generate_patch_from_files
 
 # Expose the classes with cleaner names
 ApplyPatchError = _rust_module.PyApplyPatchError
 InMemoryPatchResult = _rust_module.PyInMemoryPatchResult
 
-__version__ = "0.3.0"
+__version__ = "0.4.0"
 __all__ = [
     "apply_patch",
     "apply_patch_in_memory", 
     "parse_patch",
     "get_tool_instructions",
     "get_api_instructions",
+    "generate_patch",
+    "generate_patch_from_files",
     "ApplyPatchError",
     "InMemoryPatchResult",
 ] 
\ No newline at end of file
